@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION @ISA $DEBUG);
-$VERSION        = '4.001_049';
+$VERSION        = '4.001_050';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -84,6 +84,10 @@ require Marpa::R3::SLR;
 ( $version_result = version_ok($Marpa::R3::Scanless::R::VERSION) )
     and die 'Marpa::R3::Scanless::R::VERSION ', $version_result;
 
+require Marpa::R3::SLV;
+( $version_result = version_ok($Marpa::R3::Scanless::V::VERSION) )
+    and die 'Marpa::R3::Scanless::V::VERSION ', $version_result;
+
 require Marpa::R3::MetaAST;
 ( $version_result = version_ok($Marpa::R3::MetaAST::VERSION) )
     and die 'Marpa::R3::MetaAST::VERSION ', $version_result;
