@@ -17,7 +17,7 @@ use warnings;
 no warnings qw(recursion);
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_049';
+$VERSION        = '4.001_050';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -174,14 +174,15 @@ sub set_last_choice {
     my $choice     = $nook->[Marpa::R3::Internal::Nook::FIRST_CHOICE];
     return if $choice > $#{$and_nodes};
     if ( nook_has_semantic_cause( $asf, $nook ) ) {
+        my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
         my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
         my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
         my $and_node_id = $and_nodes->[$choice];
-        my ($current_predecessor) = $slr->call_by_tag(
+        my ($current_predecessor) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA',
-            local slr, id = ...
-            local current = slr.lmw_b:_and_node_predecessor(id)
+            local slv, id = ...
+            local current = slv.lmw_b:_and_node_predecessor(id)
             return current and current or -1
 END_OF_LUA
             'i', $and_node_id);
@@ -189,11 +190,11 @@ END_OF_LUA
             $choice++;
             $and_node_id = $and_nodes->[$choice];
             last AND_NODE if not defined $and_node_id;
-            my ($next_predecessor) = $slr->call_by_tag(
+            my ($next_predecessor) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
             <<'END_OF_LUA',
-                local slr, id = ...
-                local next = slr.lmw_b:_and_node_predecessor(id)
+                local slv, id = ...
+                local next = slv.lmw_b:_and_node_predecessor(id)
                 return next and next or -1
 END_OF_LUA
                 'i', ($and_node_id // -1));
@@ -227,16 +228,17 @@ sub nook_increment {
 sub nook_has_semantic_cause {
     my ( $asf, $nook ) = @_;
     my $or_node   = $nook->[Marpa::R3::Internal::Nook::OR_NODE];
-    my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($result) = $slr->call_by_tag(
+    my ($result) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $or_node);
-    local recce, or_node = ...
-    local irl_id = recce.lmw_b:_or_node_nrl(or_node)
-    local predot_position = recce.lmw_b:_or_node_position(or_node) - 1
-    local predot_isyid = recce.slg.g1:_nrl_rhs(irl_id, predot_position)
-    return recce.slg.g1:_nsy_is_semantic(predot_isyid)
+    local slv, or_node = ...
+    local slr = slv.slr
+    local irl_id = slv.lmw_b:_or_node_nrl(or_node)
+    local predot_position = slv.lmw_b:_or_node_position(or_node) - 1
+    local predot_isyid = slr.slg.g1:_nrl_rhs(irl_id, predot_position)
+    return slr.slg.g1:_nsy_is_semantic(predot_isyid)
 END_OF_LUA
      return $result;
 } ## end sub nook_has_semantic_cause
@@ -246,13 +248,13 @@ END_OF_LUA
 sub Marpa::R3::ASF::peak {
     my ($asf)    = @_;
     my $or_nodes = $asf->[Marpa::R3::Internal::ASF::OR_NODES];
-    my $slr      = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv      = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($augment_or_node_id) = $slr->call_by_tag(
+    my ($augment_or_node_id) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', '');
-        local recce = ...
-        local bocage = recce.lmw_b
+        local slv = ...
+        local bocage = slv.lmw_b
         if not bocage then error('No Bocage') end
         return bocage:_top_or_node()
 END_OF_LUA
@@ -260,9 +262,9 @@ END_OF_LUA
     # TODO: Why does Lua think this was a string?
     my $augment_and_node_id = $or_nodes->[$augment_or_node_id]->[0];
     my ($start_or_node_id)
-        = $slr->call_by_tag(
+        = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
-            'local recce, id = ...; return recce.lmw_b:_and_node_cause(id)',
+            'local slv, id = ...; return slv.lmw_b:_and_node_cause(id)',
             'i',
             $augment_and_node_id
             );
@@ -383,10 +385,11 @@ sub Marpa::R3::ASF::new {
     my $asf = bless [], $class;
 
     my $slr;
+    my $end_of_parse;
 
     for my $arg_hash (@arg_hashes) {
         ARG: for my $arg ( keys %{$arg_hash} ) {
-            if ( $arg eq 'slr' ) {
+            if ( $arg eq 'recognizer' ) {
                 $asf->[Marpa::R3::Internal::ASF::SLR] = $slr =
                     $arg_hash->{$arg};
                 next ARG;
@@ -396,32 +399,28 @@ sub Marpa::R3::ASF::new {
                     $arg_hash->{$arg};
                 next ARG;
             }
+            if ( $arg eq 'end' ) {
+                $end_of_parse = $arg_hash->{$arg};
+                next ARG;
+            }
             Marpa::R3::exception(
                 qq{Unknown named arg to $asf->new(): "$arg"});
         } ## end ARG: for my $arg ( keys %{$arg_hash} )
     } ## end for my $arg_hash (@arg_hashes)
 
     Marpa::R3::exception(
-        q{The "slr" named argument must be specified with the Marpa::R3::ASF::new method}
+        q{The "recognizer" named argument must be specified with the Marpa::R3::ASF::new method}
     ) if not defined $slr;
     $asf->[Marpa::R3::Internal::ASF::SLR] = $slr;
     $asf->[Marpa::R3::Internal::ASF::FACTORING_MAX] //= 42;
 
     my $slg       = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
 
-     $slr->call_by_tag( ('@' . __FILE__ . ':' . __LINE__),
-    <<'END_OF_LUA', '');
-    local recce = ...
-    if recce.tree_mode then
-        error(
-            "An attempt was made to create an ASF for a SLIF recognizer already in use\n"
-            .. "   The recognizer must be reset first\n"
-            .. string.format('  The current SLIF recognizer mode is $q\n',
-                recce.tree_mode)
-        )
-    end
-    recce.tree_mode = 'forest'
-END_OF_LUA
+    my %v_args = (recognizer => $slr);
+    $v_args{end} = $end_of_parse if $end_of_parse;
+    my $slv = Marpa::R3::Scanless::V->new(\%v_args);
+    Marpa::R3::exception( q{No parse in $asf->new()}) if not $slv;
+    $asf->[Marpa::R3::Internal::ASF::SLV] = $slv;
 
     (   $asf->[Marpa::R3::Internal::ASF::RULE_RESOLUTIONS],
         $asf->[Marpa::R3::Internal::ASF::LEXEME_RESOLUTIONS]
@@ -444,11 +443,11 @@ END_OF_LUA
 
     $asf->[Marpa::R3::Internal::ASF::GLADES] = [];
 
-    my ($is_null) = $slr->call_by_tag(
+    my ($is_null) = $slv->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', '>*' ) ;
-    local recce = ...
-    local order = recce:ordering_get()
+    local slv = ...
+    local order = slv.lmw_o
     if not order then
         error( 'Parse failed' )
     end
@@ -464,13 +463,13 @@ END_OF_LUA
     my $or_nodes = $asf->[Marpa::R3::Internal::ASF::OR_NODES] = [];
     OR_NODE: for ( my $or_node_id = 0;; $or_node_id++ ) {
 
-        my ($and_node_ids) = $slr->call_by_tag(
+        my ($and_node_ids) = $slv->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', 'i>*', $or_node_id );
         -- assumes throw mode
-        local recce, or_node_id = ...
+        local slv, or_node_id = ...
         local and_node_ids = {}
-        local order = recce.lmw_o
+        local order = slv.lmw_o
         local count = order:_or_node_and_node_count(or_node_id)
         if not count then return and_node_ids end
         for ix = 1, count do
@@ -518,14 +517,16 @@ sub Marpa::R3::ASF::glade_visited_clear {
 
 sub nid_sort_ix {
     my ( $asf, $nid ) = @_;
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
     my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
 
     if ( $nid >= 0 ) {
-        my ($result) = $slr->call_by_tag(
+        my ($result) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', 'i', $nid);
-        local slr, nid = ...
-        local irl_id = slr.lmw_b:_or_node_nrl(nid)
+        local slv, nid = ...
+        local slr = slv.slr
+        local irl_id = slv.lmw_b:_or_node_nrl(nid)
         return slr.slg.g1:_source_xrl(irl_id)
 END_OF_LUA
         return $result;
@@ -533,11 +534,12 @@ END_OF_LUA
 
     my $and_node_id  = nid_to_and_node($nid);
 
-    my ($result) = $slr->call_by_tag(
+    my ($result) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $and_node_id);
-    local slr, and_node_id = ...
-    local token_nsy_id = slr.lmw_b:_and_node_symbol(and_node_id)
+    local slv, and_node_id = ...
+    local slr = slv.slr
+    local token_nsy_id = slv.lmw_b:_and_node_symbol(and_node_id)
     local token_id = slr.slg.g1:_source_xsy(token_nsy_id)
     -- -2 is reserved for 'end of data'
     return -token_id - 3
@@ -553,23 +555,24 @@ sub Marpa::R3::ASF::grammar {
 } ## end sub Marpa::R3::ASF::grammar
 
 # TODO -- Document this method
-sub Marpa::R3::ASF::recce {
+sub Marpa::R3::ASF::recognizer {
     my ($asf)   = @_;
     my $slr     = $asf->[Marpa::R3::Internal::ASF::SLR];
     return $slr;
-} ## end sub Marpa::R3::ASF::recce
+}
 
 sub nid_rule_id {
     my ( $asf, $nid ) = @_;
     return if $nid < 0;
-    my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($xrl_id) = $slr->call_by_tag(
+    my ($xrl_id) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $nid);
-    local recce, nid = ...
-    local irl_id = recce.lmw_b:_or_node_nrl(nid)
-    local xrl_id = recce.slg.g1:_source_xrl(irl_id)
+    local slv, nid = ...
+    local slr = slv.slr
+    local irl_id = slv.lmw_b:_or_node_nrl(nid)
+    local xrl_id = slr.slg.g1:_source_xrl(irl_id)
     return xrl_id
 END_OF_LUA
     return $xrl_id;
@@ -577,14 +580,15 @@ END_OF_LUA
 
 sub or_node_es_span {
     my ( $asf, $choicepoint ) = @_;
-    my $slr        = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv        = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($origin_es, $current_es) = $slr->call_by_tag(
+    my ($origin_es, $current_es) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $choicepoint);
-    local recce, choicepoint = ...
-    local origin_es = recce.lmw_b:_or_node_origin(choicepoint)
-    local current_es = recce.lmw_b:_or_node_set(choicepoint)
+    local slv, choicepoint = ...
+    local slr = slv.slr
+    local origin_es = slv.lmw_b:_or_node_origin(choicepoint)
+    local current_es = slv.lmw_b:_or_node_set(choicepoint)
     return origin_es, current_es
 END_OF_LUA
 
@@ -593,13 +597,14 @@ END_OF_LUA
 
 sub token_es_span {
     my ( $asf, $and_node_id ) = @_;
-    my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($predecessor_id, $parent_or_node_id) = $slr->call_by_tag(
+    my ($predecessor_id, $parent_or_node_id) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA',
-        local slr, and_node_id = ...
-        local b = slr.lmw_b
+        local slv, and_node_id = ...
+        local slr = slv.slr
+        local b = slv.lmw_b
         return
             b:_and_node_predecessor(and_node_id),
             b:_and_node_parent(and_node_id)
@@ -608,11 +613,12 @@ END_OF_LUA
 
     if ( defined $predecessor_id ) {
 
-        my ($origin_es, $current_es) = $slr->call_by_tag(
+        my ($origin_es, $current_es) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA',
-            local slr, predecessor_id, parent_or_node_id = ...
-            local b = slr.lmw_b
+            local slv, predecessor_id, parent_or_node_id = ...
+            local slr = slv.slr
+            local b = slv.lmw_b
             return
                 b:_or_node_set(predecessor_id),
                 b:_or_node_set(parent_or_node_id)
@@ -659,13 +665,14 @@ sub nid_token_id {
     my ( $asf, $nid ) = @_;
     return if $nid > $NID_LEAF_BASE;
     my $and_node_id  = nid_to_and_node($nid);
-    my $slr          = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv          = $asf->[Marpa::R3::Internal::ASF::SLV];
 
-    my ($token_id) = $slr->call_by_tag(
+    my ($token_id) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA',
-        local slr, and_node_id = ...
-        local token_nsy_id = slr.lmw_b:_and_node_symbol(and_node_id)
+        local slv, and_node_id = ...
+        local slr = slv.slr
+        local token_nsy_id = slv.lmw_b:_and_node_symbol(and_node_id)
         local token_id = slr.slg.g1:_source_xsy(token_nsy_id)
         return token_id
 END_OF_LUA
@@ -681,12 +688,13 @@ sub nid_symbol_id {
     Marpa::R3::exception("No symbol ID for node ID: $nid") if $nid < 0;
 
     # Not a token, so return the LHS of the rule
-    my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
-    my ($lhs_id) = $slr->call_by_tag(
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
+    my ($lhs_id) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA',
-        local slr, nid = ...
-        local irl_id = slr.lmw_b:_or_node_nrl(nid)
+        local slv, nid = ...
+        local slr = slv.slr
+        local irl_id = slv.lmw_b:_or_node_nrl(nid)
         local g1g = slr.slg.g1
         local xrl_id = g1g:_source_xrl(irl_id)
         local lhs_id = g1g:rule_lhs(xrl_id)
@@ -824,8 +832,7 @@ sub factoring_finish {
     my $nidset_by_id   = $asf->[Marpa::R3::Internal::ASF::NIDSET_BY_ID];
     my $powerset_by_id = $asf->[Marpa::R3::Internal::ASF::POWERSET_BY_ID];
 
-    my $slr       = $asf->[Marpa::R3::Internal::ASF::SLR];
-    my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
+    my $slv       = $asf->[Marpa::R3::Internal::ASF::SLV];
 
     my @worklist = ( 0 .. $#{$factoring_stack} );
 
@@ -844,9 +851,9 @@ sub factoring_finish {
             if ( !$work_nook->[Marpa::R3::Internal::Nook::CAUSE_IS_EXPANDED] )
             {
                 if ( not nook_has_semantic_cause( $asf, $work_nook ) ) {
-                    ($child_or_node) = $slr->call_by_tag(
+                    ($child_or_node) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
-                        'local slr, work_and_node_id = ...; return slr.lmw_b:_and_node_cause(work_and_node_id)',
+                        'local slv, work_and_node_id = ...; return slv.lmw_b:_and_node_cause(work_and_node_id)',
                         'i',
                         $work_and_node_id);
                     $child_is_cause = 1;
@@ -857,9 +864,9 @@ sub factoring_finish {
             if ( !$work_nook
                 ->[Marpa::R3::Internal::Nook::PREDECESSOR_IS_EXPANDED] )
             {
-                ($child_or_node) = $slr->call_by_tag(
+                ($child_or_node) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
-                    'local slr, work_and_node_id = ...; return slr.lmw_b:_and_node_predecessor(work_and_node_id)',
+                    'local slv, work_and_node_id = ...; return slv.lmw_b:_and_node_predecessor(work_and_node_id)',
                     'i',
                     $work_and_node_id);
                 if ( defined $child_or_node ) {
@@ -903,12 +910,12 @@ sub factoring_finish {
 
 sub and_nodes_to_cause_nids {
     my ( $asf, @and_node_ids ) = @_;
-    my $slr    = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv    = $asf->[Marpa::R3::Internal::ASF::SLV];
     my %causes = ();
     for my $and_node_id (@and_node_ids) {
-        my ($cause_nid) = $slr->call_by_tag(
+        my ($cause_nid) = $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
-            'local recce, and_node_id = ...; return recce.lmw_b:_and_node_cause(and_node_id)',
+            'local slv, and_node_id = ...; return slv.lmw_b:_and_node_cause(and_node_id)',
             'i',
             $and_node_id);
         $cause_nid //= and_node_to_nid($and_node_id);
@@ -1350,8 +1357,8 @@ sub Marpa::R3::Internal::ASF::ambiguities_show {
 
             my $l0_length = $asf->glade_L0_length($glade);
             my ( $g1_start, $g1_length )       = $asf->glade_g1_span($glade);
-            my ( $l0_block1,      $l0_pos1 )       = $slr->g1_to_l0_first($g1_start);
-            my ( $l0_block2,      $l0_pos2 )       = $slr->g1_to_l0_last($g1_start + $g1_length -1);
+            my ( $l0_block1,      $l0_pos1 )       = $slr->g1_to_block_first($g1_start);
+            my ( $l0_block2,      $l0_pos2 )       = $slr->g1_to_block_last($g1_start + $g1_length -1);
             my $l0_range = $slr->lc_brief($l0_block1, $l0_pos1, $l0_block2, $l0_pos2);
             my $display_length = List::Util::min( $l0_length, 60 );
             $result
@@ -1413,8 +1420,8 @@ END_OF_LUA
                     $asf->glade_g1_span($first_downglade);
                 my $g1_last = $g1_length > 0 ? ($g1_start + $g1_length - 1) : $g1_start;
 
-                my ( $l0_block1,      $l0_pos1 )       = $slr->g1_to_l0_first($g1_start);
-                my ( $l0_block2,      $l0_pos2 )       = $slr->g1_to_l0_last($g1_last);
+                my ( $l0_block1,      $l0_pos1 )       = $slr->g1_to_block_first($g1_start);
+                my ( $l0_block2,      $l0_pos2 )       = $slr->g1_to_block_last($g1_last);
                 my $l0_range = $slr->lc_brief($l0_block1, $l0_pos1, $l0_block2, $l0_pos2);
 
                 my $first_length = $asf->glade_L0_length($first_downglade);
@@ -1457,7 +1464,7 @@ END_OF_LUA
                     }
 
                     my ( $g1_start, $g1_length ) = $asf->glade_g1_span($glade_id);
-                    my ( $l0_block,      $l0_pos )       = $slr->g1_to_l0_last($g1_start + $g1_length -1);
+                    my ( $l0_block,      $l0_pos )       = $slr->g1_to_block_last($g1_start + $g1_length -1);
                     my $l0_location = $slr->lc_brief($l0_block, $l0_pos);
 
                     $result
@@ -1845,7 +1852,7 @@ sub Marpa::R3::ASF::show_powersets {
 
 sub dump_nook {
     my ( $asf, $nook ) = @_;
-    my $slr        = $asf->[Marpa::R3::Internal::ASF::SLR];
+    my $slv        = $asf->[Marpa::R3::Internal::ASF::SLV];
     my $or_nodes   = $asf->[Marpa::R3::Internal::ASF::OR_NODES];
     my $or_node_id = $nook->[Marpa::R3::Internal::Nook::OR_NODE];
     my $and_node_count = scalar @{ $or_nodes->[$or_node_id] };
@@ -1866,7 +1873,7 @@ sub dump_nook {
         . $nook->[Marpa::R3::Internal::Nook::FIRST_CHOICE] . q{-}
         . $nook->[Marpa::R3::Internal::Nook::LAST_CHOICE]
         . qq{ of $and_node_count: };
-    $text .= $slr->verbose_or_node($or_node_id);
+    $text .= $slv->verbose_or_node($or_node_id);
     return $text;
 } ## end sub dump_nook
 
