@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_049';
+$VERSION        = '4.001_050';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -29,24 +29,29 @@ use English qw( -no_match_vars );
 sub new {
     my ( $class, $p_rules_source ) = @_;
     my $meta_recce = Marpa::R3::Internal::Scanless::meta_recce();
+    my $valuer;
     eval { $meta_recce->read($p_rules_source) }
-        or Marpa::R3::exception( "Parse of BNF/Scanless source failed\n",
+      or Marpa::R3::exception( "Parse of BNF/Scanless source failed\n",
         $EVAL_ERROR );
-    if ( my $ambiguity_status = $meta_recce->ambiguous() ) {
+    $valuer = Marpa::R3::Scanless::V->new( { recognizer => $meta_recce } );
+    my $ambiguity_level = $valuer->ambiguity_level();
+    if ( $ambiguity_level != 1 ) {
+        my $ambiguity_status = $valuer->ambiguous();
         Marpa::R3::exception( "Parse of BNF/Scanless source failed:\n",
             $ambiguity_status );
     }
-    my $value_ref = $meta_recce->value();
+    my $value_ref = $valuer->value();
     Marpa::R3::exception('Parse of BNF/Scanless source failed')
-        if not defined $value_ref;
+      if not defined $value_ref;
     my $ast = { meta_recce => $meta_recce, top_node => ${$value_ref} };
     return bless $ast, $class;
-} ## end sub new
+}
 
 sub Marpa::R3::Internal::MetaAST::Parse::substring {
     my ( $parse, $start, $length ) = @_;
     my $meta_slr      = $parse->{meta_recce};
-    my $string        = $meta_slr->literal( $start, $length );
+    my ($block_id) = $meta_slr->block_progress();
+    my $string        = $meta_slr->literal( $block_id, $start, $length );
     chomp $string;
     return $string;
 } ## end sub Marpa::R3::Internal::MetaAST::Parse::substring
