@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_049';
+$VERSION        = '4.001_050';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -145,7 +145,7 @@ sub Marpa::R3::Scanless::G::set {
             q{Bad named argument(s) to $slg->set() method} . join q{ },
             @bad_arguments );
     }
-    return $slg;
+    return;
 }
 
 sub Marpa::R3::Internal::Scanless::G::set {
@@ -1645,11 +1645,11 @@ sub Marpa::R3::Scanless::G::l0_rules_show {
     my $verbose = $options->{verbose} or 0;
     my $diag = $options->{diag} ? 1 : 0;
     my $tag = '@' . __FILE__ . ':' . __LINE__;
-    my $code = <<'END_OF_CODE';
+    my $code = <<'END_OF_LUA';
     local slg, verbose, diag = ...
     diag = diag ~= 0 -- convert diag to a boolean
     return slg:l0_rules_show({ verbose = verbose, diag = diag })
-END_OF_CODE
+END_OF_LUA
     my ($retour) = $slg->call_by_tag($tag, $code, 'ii',
         $verbose, $diag);
     return $retour;
@@ -1682,6 +1682,11 @@ kwgen(__LINE__, qw(lmg_symbol_display_form lmg_symbol_display_form si));
 kwgen(__LINE__, qw(g1_symbol_display_form g1_symbol_display_form i));
 kwgen(__LINE__, qw(l0_symbol_display_form l0_symbol_display_form i));
 
+kwgen(__LINE__, qw(symbol_angled_form symbol_angled_form i));
+kwgen(__LINE__, qw(lmg_symbol_angled_form lmg_symbol_angled_form si));
+kwgen(__LINE__, qw(g1_symbol_angled_form g1_symbol_angled_form i));
+kwgen(__LINE__, qw(l0_symbol_angled_form l0_symbol_angled_form i));
+
 kwgen(__LINE__, qw(symbol_dsl_form symbol_dsl_form i));
 kwgen(__LINE__, qw(lmg_symbol_dsl_form lmg_symbol_dsl_form si));
 kwgen(__LINE__, qw(g1_symbol_dsl_form g1_symbol_dsl_form i));
