@@ -16,7 +16,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_049';
+$VERSION        = '4.001_050';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -26,15 +26,6 @@ package Marpa::R3::Internal::Value;
 
 use English qw( -no_match_vars );
 
-sub Marpa::R3::show_rank_ref {
-    my ($rank_ref) = @_;
-    return 'undef' if not defined $rank_ref;
-    return 'SKIP'  if $rank_ref == -1;
-    return ${$rank_ref};
-} ## end sub Marpa::R3::show_rank_ref
-
-package Marpa::R3::Internal::Value;
-
 # Given the grammar and an action name, resolve it to a closure,
 # or return undef
 sub resolve_action {
@@ -147,159 +138,6 @@ qq{Failed resolution of action "$closure_name" to $fully_qualified_name\n};
 
 }
 
-our $CONTEXT_EXCEPTION_CLASS = __PACKAGE__ . '::Context_Exception';
-
-sub Marpa::R3::Context::bail {   ## no critic (Subroutines::RequireArgUnpacking)
-    if ( scalar @_ == 1 and ref $_[0] ) {
-        die bless { exception_object => $_[0] }, $CONTEXT_EXCEPTION_CLASS;
-    }
-    my $error_string = join q{}, @_;
-    my ( $package, $filename, $line ) = caller;
-    chomp $error_string;
-    die bless { message => qq{User bailed at line $line in file "$filename"\n}
-          . $error_string
-          . "\n" }, $CONTEXT_EXCEPTION_CLASS;
-} ## end sub Marpa::R3::Context::bail
-## use critic
-
-sub Marpa::R3::Context::g1_range {
-    my $slr = $Marpa::R3::Context::slr;
-    my ( $start, $end ) =
-      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
-local slr = ...
-return slr.this_step.start_es_id, slr.this_step.es_id
-END_OF_LUA
-    return $start, $end;
-} ## end sub Marpa::R3::Context::g1_range
-
-sub Marpa::R3::Context::lc_range {
-    my $slr = $Marpa::R3::Context::slr;
-    my ( $lc_range ) =
-      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
-local slr = ...
-local g1_first = slr.this_step.start_es_id
-local g1_last = slr.this_step.es_id - 1
-local l0_first_b, l0_first_p = slr:g1_pos_to_l0_first(g1_first)
-local l0_last_b, l0_last_p = slr:g1_pos_to_l0_last(g1_last)
-return slr:lc_range_brief(l0_first_b, l0_first_p, l0_last_b, l0_last_p)
-END_OF_LUA
-    return $lc_range;
-}
-
-sub Marpa::R3::Context::g1_span {
-    my $slr = $Marpa::R3::Context::slr;
-    my ( $start, $length ) =
-      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
-local slr = ...
-local start = slr.this_step.start_es_id + 0
-local length = (start - slr.this_step.es_id) + 1
-return start, length
-END_OF_LUA
-    return $start, $length;
-}
-
-sub code_problems {
-    my $args = shift;
-
-    my $grammar;
-    my $fatal_error;
-    my $warnings = [];
-    my $where    = '?where?';
-    my $long_where;
-    my @msg = ();
-    my $eval_value;
-    my $eval_given = 0;
-
-    push @msg, q{=} x 60, "\n";
-  ARG: for my $arg ( keys %{$args} ) {
-        my $value = $args->{$arg};
-        if ( $arg eq 'fatal_error' ) { $fatal_error = $value; next ARG }
-        if ( $arg eq 'grammar' )     { $grammar     = $value; next ARG }
-        if ( $arg eq 'where' )       { $where       = $value; next ARG }
-        if ( $arg eq 'long_where' )  { $long_where  = $value; next ARG }
-        if ( $arg eq 'warnings' )    { $warnings    = $value; next ARG }
-        if ( $arg eq 'eval_ok' ) {
-            $eval_value = $value;
-            $eval_given = 1;
-            next ARG;
-        }
-        push @msg, "Unknown argument to code_problems: $arg";
-    } ## end ARG: for my $arg ( keys %{$args} )
-
-  GIVEN_FATAL_ERROR_REF_TYPE: {
-        my $fatal_error_ref_type = ref $fatal_error;
-        last GIVEN_FATAL_ERROR_REF_TYPE if not $fatal_error_ref_type;
-        if ( $fatal_error_ref_type eq $CONTEXT_EXCEPTION_CLASS ) {
-            my $exception_object = $fatal_error->{exception_object};
-            die $exception_object if defined $exception_object;
-            my $exception_message = $fatal_error->{message};
-            die $exception_message if defined $exception_message;
-            die "Internal error: bad $CONTEXT_EXCEPTION_CLASS object";
-        } ## end if ( $fatal_error_ref_type eq $CONTEXT_EXCEPTION_CLASS)
-        $fatal_error =
-            "Exception thrown as object inside Marpa closure\n"
-          . ( q{ } x 4 )
-          . "This is not allowed\n"
-          . ( q{ } x 4 )
-          . qq{Exception as string is "$fatal_error"};
-    } ## end GIVEN_FATAL_ERROR_REF_TYPE:
-
-    my @problem_line     = ();
-    my $max_problem_line = -1;
-    for my $warning_data ( @{$warnings} ) {
-        my ( $warning, $package, $filename, $problem_line ) = @{$warning_data};
-        $problem_line[$problem_line] = 1;
-        $max_problem_line = List::Util::max $problem_line, $max_problem_line;
-    } ## end for my $warning_data ( @{$warnings} )
-
-    $long_where //= $where;
-
-    my $warnings_count = scalar @{$warnings};
-    {
-        my @problems;
-        my $false_eval = $eval_given && !$eval_value && !$fatal_error;
-        if ($false_eval) {
-            push @problems, '* THE MARPA SEMANTICS RETURNED A PERL FALSE',
-              'Marpa::R3 requires its semantics to return a true value';
-        }
-        if ($fatal_error) {
-            push @problems, '* THE MARPA SEMANTICS PRODUCED A FATAL ERROR';
-        }
-        if ($warnings_count) {
-            push @problems,
-              "* THERE WERE $warnings_count WARNING(S) IN THE MARPA SEMANTICS:",
-              'Marpa treats warnings as fatal errors';
-        }
-        if ( not scalar @problems ) {
-            push @msg, '* THERE WAS A FATAL PROBLEM IN THE MARPA SEMANTICS';
-        }
-        push @msg, ( join "\n", @problems ) . "\n";
-    }
-
-    push @msg, "* THIS IS WHAT MARPA WAS DOING WHEN THE PROBLEM OCCURRED:\n"
-      . $long_where . "\n";
-
-    for my $warning_ix ( 0 .. ( $warnings_count - 1 ) ) {
-        push @msg, "* WARNING MESSAGE NUMBER $warning_ix:\n";
-        my $warning_message = $warnings->[$warning_ix]->[0];
-        $warning_message =~ s/\n*\z/\n/xms;
-        push @msg, $warning_message;
-    } ## end for my $warning_ix ( 0 .. ( $warnings_count - 1 ) )
-
-    if ($fatal_error) {
-        push @msg, "* THIS WAS THE FATAL ERROR MESSAGE:\n";
-        my $fatal_error_message = $fatal_error;
-        $fatal_error_message =~ s/\n*\z/\n/xms;
-        push @msg, $fatal_error_message;
-    } ## end if ($fatal_error)
-
-    Marpa::R3::exception(@msg);
-
-    # this is to keep perlcritic happy
-    return 1;
-
-} ## end sub code_problems
-
 # Dump semantics for diagnostics
 sub Marpa::R3::Scanless::R::show_semantics {
     my ( $slg, @ops ) = @_;
@@ -540,75 +378,6 @@ END_OF_LUA
     return ( $rule_resolutions, \@lexeme_resolutions );
 }
 
-sub do_tree_ops {
-    my ( $slr, $tree ) = @_;
-    my $blessing = Scalar::Util::blessed $tree;
-    if ( not defined $blessing ) {
-        my $ref = ref $tree;
-
-        # say STDERR "ref_type = $ref";
-        if ( $ref eq 'ARRAY' ) {
-
-            # say STDERR "Recursing into unblessed array";
-            return [ map { do_tree_ops( $slr, $_ ) } @{$tree} ];
-        }
-        if ( $ref eq 'REF' ) {
-
-            # say STDERR "Recursing into unblessed ref";
-            return \( do_tree_ops( $slr, ${$tree} ) );
-        }
-        return $tree;
-    }
-    if ( $blessing ne "Marpa::R3::Tree_Op" ) {
-        my $ref_type = Scalar::Util::reftype $tree;
-
-        # say STDERR "ref_type = $ref_type";
-        if ( $ref_type eq 'ARRAY)' ) {
-
-            # say STDERR "Recursing into blessed array";
-            return bless [ map { do_tree_ops( $slr, $_ ) } @{$tree} ],
-              $blessing;
-        }
-        if ( $ref_type eq 'REF)' ) {
-
-            # say STDERR "Recursing into blessed ref";
-            return bless \( do_tree_ops( $slr, ${$tree} ) ), $blessing;
-        }
-        return $tree;
-    }
-    my $tree_op = $tree->[0];
-    Marpa::R3::exception("Tree op missing") if not defined $tree_op;
-    if ( $tree_op eq 'asis' ) {
-
-        # say STDERR "Removing asis wrapper";
-        return $tree->[1];
-    }
-    if ( $tree_op ne 'perl' ) {
-        Marpa::R3::exception(qq{Unknown tree op ("$tree_op")});
-    }
-    my $lua_to_perl_tree_op = $tree->[1];
-    if ( $lua_to_perl_tree_op eq 'bless' ) {
-        my $constant_ix = $tree->[3];
-        my $slg         = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-        my $blessing = $slg->[Marpa::R3::Internal::Scanless::G::CONSTANTS]
-          ->[$constant_ix];
-        return bless do_tree_ops( $slr, $tree->[2] ), $blessing;
-    }
-    if ( $lua_to_perl_tree_op eq 'constant' ) {
-        my $constant_ix = $tree->[2];
-        my $slg         = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-        return $slg->[Marpa::R3::Internal::Scanless::G::CONSTANTS]
-          ->[$constant_ix];
-    }
-    if ( $lua_to_perl_tree_op eq 'undef' ) {
-        my $constant_ix = $tree->[2];
-        my $slg         = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-        return undef;
-    }
-    Marpa::R3::exception(
-        qq{Unknown Lua-to-Perl tree op ("$lua_to_perl_tree_op")});
-}
-
 sub op_fn_key_by_name {
     my ( $slg, $name ) = @_;
     my ($key) = $slg->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -1468,490 +1237,24 @@ END_OF_LUA
 
 }
 
-# Returns false if no parse
 sub Marpa::R3::Scanless::R::value {
     my ( $slr, $per_parse_arg ) = @_;
-    my $slg    = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-
-    my $trace_actions =
-      $slg->[Marpa::R3::Internal::Scanless::G::TRACE_ACTIONS] // 0;
-    my $trace_file_handle =
-      $slr->[Marpa::R3::Internal::Scanless::R::TRACE_FILE_HANDLE];
-
-    if ( scalar @_ != 1 ) {
-        Marpa::R3::exception(
-            'Too many arguments to Marpa::R3::Scanless::R::value')
-          if ref $slr ne 'Marpa::R3::Scanless::R';
+    my $slv = Marpa::R3::Scanless::V->new( { recognizer => $slr } );
+    my $ambiguity_level = $slv->ambiguity_level();
+    return if $ambiguity_level == 0;
+    if ( $ambiguity_level != 1 ) {
+        my $ambiguous_status = $slv->ambiguous();
+        Marpa::R3::exception( "Parse of the input is ambiguous\n",
+            $ambiguous_status );
     }
-
-    my ($result) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        << 'END_OF_LUA', '>*' );
-        local slr=...
-        local g1r = slr.g1
-
-        slr.tree_mode = slr.tree_mode or 'tree'
-        if slr.tree_mode ~= 'tree' then
-            error(
-                "value() called when recognizer is not in tree mode\n"
-                .. string.format('  The current mode is %q\n', slr.tree_mode)
-            )
-        end
-
-        slr.phase = 'value'
-        local furthest_earleme = g1r:furthest_earleme()
-        local last_completed_earleme = g1r:current_earleme()
-        if furthest_earleme ~= last_completed_earleme then
-            error(string.format(
-                "Attempt to evaluate incompletely recognized parse:\n"
-                .. "  Last token ends at location %d\n"
-                .. "  Recognition done only as far as location %d\n",
-                furthest_earleme,
-                last_completed_earleme
-            ))
-        end
-        local lmw_t = slr.lmw_t
-        if not lmw_t then
-            -- No tree, therefore ordering is not initialized
-            local lmw_o = slr:ordering_get()
-            if not lmw_o then return false end
-            lmw_t = _M.tree_new(lmw_o)
-            slr.lmw_t = lmw_t
-        end
-
-        local max_parses = slr.max_parses
-        local parse_count = slr.lmw_t:parse_count()
-        if max_parses and parse_count > max_parses then
-            error(string.format("Maximum parse count (%d) exceeded", max_parses));
-        end
-        -- io.stderr:write('tree:', inspect(slr.lmw_t))
-        slr.lmw_v = nil
-        -- print(inspect(_G))
-        collectgarbage()
-        local result = slr.lmw_t:next()
-        if not result then return result end
-        -- print('result:', result)
-        slr.lmw_v = _M.value_new(slr.lmw_t)
-    return result
-END_OF_LUA
-
-    return if not defined $result;
-
-    local $Marpa::R3::Context::rule = undef;
-    local $Marpa::R3::Context::irlid = undef;
-    local $Marpa::R3::Context::slr  = $slr;
-    local $Marpa::R3::Context::slg =
-      $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-
-    my ($trace_values) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        << 'END_OF_LUA', '' );
-    local slr = ...
-    local trace_values = slr.trace_values or 0
-    slr.lmw_v:_trace(trace_values)
-    slr:value_init(trace_values)
-    return trace_values
-END_OF_LUA
-
-  STEP: while (1) {
-        my ( $value_type, @value_data ) = $slr->call_by_tag(
-            ( '@' . __FILE__ . ':' . __LINE__ ),
-            << 'END_OF_LUA', '');
-          local recce = ...
-          local result, new_values = recce:find_and_do_ops()
-          if result == -1 then return 'trace', -1, {} end
-          local this = recce.this_step
-          local step_type = this.type
-          if step_type == 'MARPA_STEP_INACTIVE' then
-             return step_type, -1, {}
-          end
-          local parm2 = -1
-          if step_type == 'MARPA_STEP_RULE' then parm2 = this.rule end
-          if step_type == 'MARPA_STEP_TOKEN' then parm2 = this.symbol end
-          if step_type == 'MARPA_STEP_NULLING_SYMBOL' then parm2 = this.symbol end
-          return step_type, parm2, new_values
-END_OF_LUA
-
-        if ($trace_values) {
-          EVENT: for ( my $event_ix = 0 ; ; $event_ix++ ) {
-                my @event = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                    <<'END_OF_LUA', 'i>*', $event_ix );
-local recce, event_ix = ...;
-local entry = recce.trace_values_queue[event_ix+1]
-if entry == nil then return end
-return table.unpack(entry)
-END_OF_LUA
-
-                # say STDERR join "!", "Lua event:", @event;
-
-                my ( $event_type, @event_data ) = @event;
-                last EVENT if not $event_type;
-                if ( $event_type eq 'MARPA_STEP_TOKEN' ) {
-                    my ( $token_id, $token_value_ix, $token_value ) =
-                      @event_data;
-                    trace_token_evaluation( $slr, $token_id, $token_value );
-                    next EVENT;
-                } ## end if ( $event_type eq 'MARPA_STEP_TOKEN' )
-
-                say {$trace_file_handle} join q{ },
-                  'value event:',
-                  map { $_ // 'undef' } $event_type, @event_data
-                  or Marpa::R3::exception('say to trace handle failed');
-            } ## end EVENT: while (1)
-
-            if ( $trace_values >= 9 ) {
-
-                my ($highest_index) =
-                  $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                    << 'END_OF_LUA', '' );
-    local recce =...
-    return recce:stack_top_index()
-END_OF_LUA
-
-                for my $i ( reverse 1 .. $highest_index ) {
-
-                    my ($value) =
-                      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                        << 'END_OF_LUA', 'i>*', $i );
-    local recce, ix =...
-    return recce:stack_get(ix)
-END_OF_LUA
-
-                    printf {$trace_file_handle} "Stack position %3d:\n", $i,
-                      or Marpa::R3::exception('print to trace handle failed');
-                    print {$trace_file_handle} q{ },
-                      Data::Dumper->new( [ \$value ] )->Terse(1)->Dump
-                      or Marpa::R3::exception('print to trace handle failed');
-                } ## end for my $i ( reverse 0 .. $value->highest_index )
-            } ## end if ( $trace_values >= 9 )
-
-        } ## end if ($trace_values)
-
-        last STEP if not defined $value_type;
-        last STEP if $value_type eq 'MARPA_STEP_INACTIVE';
-        next STEP if $value_type eq 'trace';
-
-        my $semantics_arg0 = $per_parse_arg // {};
-        my $null_values = $slg->[Marpa::R3::Internal::Scanless::G::NULL_VALUES];
-        my $nulling_closures =
-          $slg->[Marpa::R3::Internal::Scanless::G::CLOSURE_BY_SYMBOL_ID];
-        my $rule_closures =
-          $slg->[Marpa::R3::Internal::Scanless::G::CLOSURE_BY_RULE_ID];
-
-        if ( $value_type eq 'MARPA_STEP_NULLING_SYMBOL' ) {
-            my ($token_id) = @value_data;
-            my $value_ref = $nulling_closures->[$token_id];
-            my $result;
-
-            my @warnings;
-            my $eval_ok;
-
-          DO_EVAL: {
-                local $SIG{__WARN__} = sub {
-                    push @warnings, [ $_[0], ( caller 0 ) ];
-                };
-
-                $eval_ok = eval {
-                    my $irlid = $null_values->[$token_id];
-                    local $Marpa::R3::Context::irlid = $irlid;
-                    local $Marpa::R3::Context::production_id =
-                      $slg->g1_rule_to_production_id($irlid);
-                    $result = $value_ref->( $semantics_arg0, [] );
-                    1;
-                };
-
-            } ## end DO_EVAL:
-
-            if ( not $eval_ok or @warnings ) {
-                my $fatal_error = $EVAL_ERROR;
-                code_problems(
-                    {
-                        fatal_error => $fatal_error,
-                        eval_ok     => $eval_ok,
-                        warnings    => \@warnings,
-                        where       => 'computing value',
-                        long_where  => 'Computing value for null symbol: '
-                          . $slg->g1_symbol_display($token_id),
-                    }
-                );
-            } ## end if ( not $eval_ok or @warnings )
-
-            my $wrapped_result = bless [ 'asis', $result ],
-              "Marpa::R3::Tree_Op";
-
-            # my $wrapped_result = $result;
-
-            $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                << 'END_OF_LUA', 'S', $wrapped_result );
-    local recce, sv =...
-    local ix = recce:stack_top_index()
-    return recce:stack_set(ix, sv)
-END_OF_LUA
-
-            trace_token_evaluation( $slr, $token_id, \$result )
-              if $trace_values;
-            next STEP;
-        } ## end if ( $value_type eq 'MARPA_STEP_NULLING_SYMBOL' )
-
-        if ( $value_type eq 'MARPA_STEP_RULE' ) {
-            my ( $irlid, $values ) = @value_data;
-            my $closure = $rule_closures->[$irlid];
-
-            next STEP if not defined $closure;
-            my $result;
-
-            {
-                my @warnings;
-                my $eval_ok;
-                local $SIG{__WARN__} = sub {
-                    push @warnings, [ $_[0], ( caller 0 ) ];
-                };
-                local $Marpa::R3::Context::irlid = $irlid;
-                local $Marpa::R3::Context::production_id = $slg->g1_rule_to_production_id($irlid);
-
-                # say STDERR "Before tree ops: ", Data::Dumper::Dumper($values);
-                $values = do_tree_ops( $slr, $values );
-
-                # say STDERR "After tree ops: ", Data::Dumper::Dumper($values);
-
-                $eval_ok = eval {
-                    $result = $closure->( $semantics_arg0, $values );
-                    1;
-                };
-
-                if ( not $eval_ok or @warnings ) {
-                    my $fatal_error = $EVAL_ERROR;
-                    code_problems(
-                        {
-                            fatal_error => $fatal_error,
-                            eval_ok     => $eval_ok,
-                            warnings    => \@warnings,
-                            where       => 'computing value',
-                            long_where  => 'Computing value for rule: '
-                              . $slg->g1_rule_show($irlid),
-                        }
-                    );
-                } ## end if ( not $eval_ok or @warnings )
-            }
-
-            # say STDERR "Before wrapping: ", Data::Dumper::Dumper($result);
-            my $wrapped_result = bless [ 'asis', $result ],
-              "Marpa::R3::Tree_Op";
-
-            # my $wrapped_result = $result;
-
-            $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                << 'END_OF_LUA', 'S', $wrapped_result );
-    local recce, sv =...
-    local index = recce:stack_top_index()
-    return recce:stack_set(index, sv)
-END_OF_LUA
-
-            if ($trace_values) {
-                say {$trace_file_handle}
-                  trace_stack_1( $slr, $values, $irlid )
-                  or Marpa::R3::exception('Could not print to trace file');
-                print {$trace_file_handle}
-                  'Calculated and pushed value: ',
-                  Data::Dumper->new( [$result] )->Terse(1)->Dump
-                  or Marpa::R3::exception('print to trace handle failed');
-            } ## end if ($trace_values)
-
-            next STEP;
-
-        } ## end if ( $value_type eq 'MARPA_STEP_RULE' )
-
-        if ( $value_type eq 'MARPA_STEP_TRACE' ) {
-
-            if ( my $trace_output = trace_op($slr) ) {
-                print {$trace_file_handle} $trace_output
-                  or Marpa::R3::exception('Could not print to trace file');
-            }
-
-            next STEP;
-
-        } ## end if ( $value_type eq 'MARPA_STEP_TRACE' )
-
-        die "Internal error: Unknown value type $value_type";
-
-    } ## end STEP: while (1)
-
-    my ($final_value) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        << 'END_OF_LUA', '', );
-    local recce =...
-    return recce:stack_get(1)
-END_OF_LUA
-
-# say "final value: ", Data::Dumper::Dumper( \(do_tree_ops($slr, $final_value)) );
-    return do_tree_ops( $slr, \($final_value) );
-
+    my $value_ref = $slv->value($per_parse_arg);
+    Marpa::R3::exception("$slr->value(): No parse\n")
+      if not $value_ref;
+    return $value_ref;
 }
 
 # INTERNAL OK AFTER HERE _marpa_
 
-sub Marpa::R3::Scanless::R::and_node_tag {
-    my ( $slr, $and_node_id ) = @_;
-
-    my ($tag) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        << 'END_OF_LUA', 'i', $and_node_id );
-    local recce,and_node_id=...
-    return recce:and_node_tag(and_node_id)
-END_OF_LUA
-
-    return $tag;
-}
-
-sub trace_token_evaluation {
-    my ( $slr, $token_id, $token_value ) = @_;
-    my $slg    = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    my $trace_file_handle =
-      $slr->[Marpa::R3::Internal::Scanless::R::TRACE_FILE_HANDLE];
-
-    my ( $nook_ix, $and_node_id ) =
-      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        << 'END_OF_LUA', '>*' );
-    recce = ...
-    local nook_ix = recce.lmw_v:_nook()
-    local o = recce.lmw_o
-    local t = recce.lmw_t
-    local or_node_id = t:_nook_or_node(nook_ix)
-    local choice = t:_nook_choice(nook_ix)
-    local and_node_id = o:_and_node_order_get( or_node_id, choice )
-    return nook_ix, and_node_id
-END_OF_LUA
-
-    if ( not defined $nook_ix ) {
-        print {$trace_file_handle} "Nulling valuator\n"
-          or Marpa::R3::exception('Could not print to trace file');
-        return;
-    }
-    my $token_name;
-    if ( defined $token_id ) {
-        $token_name = $slg->g1_symbol_display($token_id);
-    }
-
-    print {$trace_file_handle}
-      'Pushed value from ',
-      $slr->and_node_tag($and_node_id),
-      ': ',
-      ( $token_name ? qq{$token_name = } : q{} ),
-      Data::Dumper->new( [ \$token_value ] )->Terse(1)->Dump
-      or Marpa::R3::exception('print to trace handle failed');
-
-    return;
-
-} ## end sub trace_token_evaluation
-
-sub trace_stack_1 {
-    my ( $slr, $args, $rule_id ) = @_;
-    my $slg    = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-
-    my $argc = scalar @{$args};
-    my ( $nook_ix, $and_node_id ) =
-      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
-    local slr = ...
-    local nook_ix = slr.lmw_v:_nook()
-    local o = slr.lmw_o
-    local t = slr.lmw_t
-    local or_node_id = t:_nook_or_node(nook_ix)
-    local choice = t:_nook_choice(nook_ix)
-    local and_node_id = o:_and_order_get(or_node_id, choice)
-    return nook_ix, and_node_id
-END_OF_LUA
-
-    return 'Popping ', $argc,
-      ' values to evaluate ',
-      $slr->and_node_tag($and_node_id),
-      ', rule: ', $slg->g1_rule_show($rule_id);
-
-} ## end sub trace_stack_1
-
-sub trace_op {
-
-    my ($slr)  = @_;
-    my $slg    = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    my $trace_output = q{};
-
-    my (
-        $cmd,
-        $nook_ix,      $or_node_id,       $choice,      $and_node_id,
-        $trace_irl_id, $virtual_rhs, $virtual_lhs,
-        $real_symbol_count
-      )
-      = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        <<'END_OF_LUA' , '' );
-    local slr = ...
-    if not slr.trace_values or slr.trace_values < 2 then
-        return 'return'
-    end
-    local nook_ix = slr.lmw_v:_nook()
-    local b = slr.lmw_b
-    local o = slr.lmw_o
-    local t = slr.lmw_t
-    local g1g = slr.slg.g1
-    local or_node_id = t:_nook_or_node(nook_ix)
-    local choice = t:_nook_choice(nook_ix)
-    local trace_irl_id = b:_or_node_nrl(or_node_id)
-    local or_node_position = b:_or_node_position(or_node_id)
-    local irl_length = g1g:_nrl_length(trace_irl_id)
-    if irl_length ~= or_node_position then
-        return 'return'
-    end
-    return
-        '', nook_ix, or_node_id, choice,
-            o:_and_order_get(or_node_id, choice), 
-            trace_irl_id,
-            g1g:_nrl_is_virtual_rhs(trace_irl_id),
-            g1g:_nrl_is_virtual_lhs(trace_irl_id),
-            g1g:_real_symbol_count(trace_irl_id)
-END_OF_LUA
-
-    if ($cmd eq 'return') { return $trace_output; }
-    return $trace_output if not $virtual_rhs and not $virtual_lhs;
-
-    if ( $virtual_rhs and not $virtual_lhs ) {
-
-        $trace_output .= join q{},
-          'Head of Virtual Rule: ',
-          $slr->and_node_tag($and_node_id),
-          ', rule: ', $slg->brief_nrl($trace_irl_id),
-          "\n",
-          'Incrementing virtual rule by ', $real_symbol_count, ' symbols', "\n"
-          or Marpa::R3::exception('Could not print to trace file');
-
-        return $trace_output;
-
-    } ## end if ( $virtual_rhs and not $virtual_lhs )
-
-    if ( $virtual_lhs and $virtual_rhs ) {
-
-        $trace_output .= join q{},
-          'Virtual Rule: ',
-          $slr->and_node_tag($and_node_id),
-          ', rule: ', $slg->brief_nrl($trace_irl_id),
-          "\nAdding ",
-          $real_symbol_count,
-          "\n";
-
-        return $trace_output;
-
-    } ## end if ( $virtual_lhs and $virtual_rhs )
-
-    if ( not $virtual_rhs and $virtual_lhs ) {
-
-        $trace_output .= join q{},
-          'New Virtual Rule: ',
-          $slr->and_node_tag($and_node_id),
-          ', rule: ', $slg->brief_nrl($trace_irl_id),
-          "\nReal symbol count is ",
-          $real_symbol_count,
-          "\n";
-
-        return $trace_output;
-
-    } ## end if ( not $virtual_rhs and $virtual_lhs )
-
-    return $trace_output;
-} ## end sub trace_op
-
 1;
 
 # vim: expandtab shiftwidth=4:
